﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.ParameterConfigurationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Problems.MetaOptimization {
  [Item("ConstrainedTypeValue", "Represents a type with constraints.")]
  [StorableClass]
  public class ConstrainedTypeValue : TypeValue {
    private IEnumerable<Type> validTypes;
    public IEnumerable<Type> ValidTypes {
      get { return validTypes; }
      set { validTypes = value; }
    }

    public override Type Value {
      set {
        if (!ValidTypes.Contains(value)) throw new NotSupportedException("Value cannot be set. Type is not enlisted in ValidTypes");
        base.Value = value;
      }
    }

    #region Constructors and Cloning
    [StorableConstructor]
    protected ConstrainedTypeValue(bool deserializing) : base(deserializing) { }
    protected ConstrainedTypeValue(ConstrainedTypeValue original, Cloner cloner)
      : base(original, cloner) {
      if (original.validTypes != null)
        this.validTypes = new List<Type>(original.validTypes);
    }
    public ConstrainedTypeValue()
      : base() {
      this.readOnly = false;
    }
    public ConstrainedTypeValue(Type value)
      : this() {
      this.Value = value;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ConstrainedTypeValue(this, cloner);
    }
    #endregion
  }

  [Item("ConstrainedTypeValue<>", "Represents a type with constraints.")]
  [StorableClass]
  public class ConstrainedTypeValue<T> : ConstrainedTypeValue where T : class, IItem {
    [StorableConstructor]
    protected ConstrainedTypeValue(bool deserializing)
      : base(deserializing) {
      if (deserializing) {
        this.ValidTypes = ApplicationManager.Manager.GetTypes(typeof(T), true).ToList();
      }
    }
    protected ConstrainedTypeValue(ConstrainedTypeValue original, Cloner cloner) : base(original, cloner) { }
    public ConstrainedTypeValue()
      : base() {
      this.ValidTypes = ApplicationManager.Manager.GetTypes(typeof(T), true).ToList();
      this.Value = ValidTypes.First();
    }
    public ConstrainedTypeValue(Type value)
      : base() {
      this.ValidTypes = ApplicationManager.Manager.GetTypes(typeof(T), true).ToList();
      this.Value = value;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ConstrainedTypeValue<T>(this, cloner);
    }
  }
}